@@ -28,30 +28,16 @@ class BenchmarkOperatorWorkloads(BenchmarkOperatorWorkloadsOperations):
         workload_module = importlib.import_module(f'benchmark_runner.benchmark_operator.{workload}')
 
         try:
-            _initialize_workload = getattr(workload_module, "initialize_workload")
-            initialize_workload = lambda: _initialize_workload(self)
-        except AttributeError:
-            logger.info(f"{workload} module has no initialize_workload method. Using the default one.")
-            initialize_workload = self.initialize_workload
-
-        try:
-            _finalize_workload = getattr(workload_module, "finalize_workload")
-            finalize_workload = lambda: _finalize_workload(self)
-        except AttributeError:
-            logger.info(f"{workload} module has no finalize_workload method. Using the default one.")
-            finalize_workload = self.finalize_workload
-
-        try:
-            initialize_workload()
+            self.initialize_workload()
             success = True
             for cls in inspect.getmembers(workload_module, inspect.isclass):
                 if workload.replace('_', '').lower() == cls[0].lower():
                     if cls[1]().run() == False:
                         success = False
 
-            finalize_workload()
+            self.finalize_workload()
 
             return success
         except Exception as err:
-            finalize_workload()
+            self.finalize_workload()
             raise err
