@@ -8,10 +8,11 @@ from typeguard import typechecked
 
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp, logger
 from benchmark_runner.common.oc.oc import OC
+from benchmark_runner.common.virtctl.virtctl import Virtctl
 from benchmark_runner.common.template_operations.template_operations import TemplateOperations
 from benchmark_runner.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
 from benchmark_runner.common.ssh.ssh import SSH
-from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import ODFNonInstalled
+from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import ODFNonInstalled, EmptyLSOPath
 from benchmark_runner.main.environment_variables import environment_variables
 from benchmark_runner.common.clouds.shared.s3.s3_operations import S3Operations
 from benchmark_runner.common.prometheus.prometheus_snapshot import PrometheusSnapshot
@@ -32,10 +33,12 @@ class BenchmarkOperatorWorkloadsOperations:
         self._run_type = self._environment_variables_dict.get('run_type', '')
         self._workloads_odf_pvc = list(self._environment_variables_dict.get('workloads_odf_pvc', ''))
         self._odf_pvc = self._environment_variables_dict.get('odf_pvc', '')
+        self._storage_type = self._environment_variables_dict.get('storage_type', '')
         self._system_metrics = self._environment_variables_dict.get('system_metrics', '')
         self._elasticsearch = self._environment_variables_dict.get('elasticsearch', '')
         self._run_artifacts = self._environment_variables_dict.get('run_artifacts', '')
         self._run_artifacts_path = self._environment_variables_dict.get('run_artifacts_path', '')
+        self._lso_path = self._environment_variables_dict.get('lso_path', '')
         self._date_key = self._environment_variables_dict.get('date_key', '')
         self._key = self._environment_variables_dict.get('key', '')
         self._endpoint_url = self._environment_variables_dict.get('endpoint_url', '')
@@ -66,6 +69,7 @@ class BenchmarkOperatorWorkloadsOperations:
         self._template = TemplateOperations(workload=self._workload)
         # set oc login
         self._oc = self.set_login(kubeadmin_password=self._kubeadmin_password)
+        self._virtctl = Virtctl()
         # PrometheusSnapshot
         if self._enable_prometheus_snapshot:
             self._snapshot = PrometheusSnapshot(oc=self._oc, artifacts_path=self._run_artifacts_path, verbose=True)
@@ -142,7 +146,7 @@ class BenchmarkOperatorWorkloadsOperations:
         @return:
         """
         # delete benchmark-operator pod if exist
-        if self._oc._is_pod_exist(pod_name='benchmark-controller-manager'):
+        if self._oc.pod_exists(pod_name='benchmark-controller-manager'):
             logger.info('make undeploy benchmark operator running pod')
             self.make_undeploy_benchmark_controller_manager(runner_path=runner_path)
 
@@ -177,10 +181,11 @@ class BenchmarkOperatorWorkloadsOperations:
         self.make_undeploy_benchmark_controller_manager(runner_path=environment_variables.environment_variables_dict['runner_path'])
 
     @logger_time_stamp
-    def system_metrics_collector(self, workload: str):
+    def system_metrics_collector(self, workload: str, es_fetch_min_time: int = None):
         """
         This method run system metrics collector
         @param workload: the workload
+        @param es_fetch_min_time:
         :return:
         """
         if self._run_type == 'test_ci':
@@ -192,8 +197,11 @@ class BenchmarkOperatorWorkloadsOperations:
         self._oc.wait_for_pod_completed(label='app=system-metrics-collector', workload=workload)
         # verify that data upload to elastic search
         if self._es_host:
-            # workload: system metrics, in case of uperf take timestamp and not uperf_ts
-            self._verify_elasticsearch_data_uploaded(index=es_index, uuid=self._oc.get_long_uuid(workload=workload), workload='system-metrics', fast_check=True)
+            # es_fetch_min_time: if we need to fetch more time
+            if es_fetch_min_time:
+                self._verify_elasticsearch_data_uploaded(index=es_index, uuid=self._oc.get_long_uuid(workload=workload), workload='system-metrics', fast_check=True, es_fetch_min_time=es_fetch_min_time)
+            else:
+                self._verify_elasticsearch_data_uploaded(index=es_index, uuid=self._oc.get_long_uuid(workload=workload), workload='system-metrics', fast_check=True)
 
     def __get_metadata(self, kind: str = None, database: str = None, status: str = None, run_artifacts_url: str = None, uuid: str = None) -> dict:
         """
@@ -214,7 +222,8 @@ class BenchmarkOperatorWorkloadsOperations:
                     'ci_date': datetime.datetime.now().strftime(date_format),
                     'pin_node_benchmark_operator': self._pin_node_benchmark_operator,
                     'pin_node1': self._pin_node1,
-                    'pin_node2': self._pin_node2}
+                    'pin_node2': self._pin_node2,
+                    'storage_type': self._storage_type}
         if kind:
             metadata.update({'kind': kind})
         if status:
@@ -262,7 +271,7 @@ class BenchmarkOperatorWorkloadsOperations:
         """
         self.__es_operations.upload_to_elasticsearch(index=index, data=self.__get_metadata(kind=kind, database=database, status=status, run_artifacts_url=run_artifacts_url, uuid=uuid))
 
-    def _verify_elasticsearch_data_uploaded(self, index: str, uuid: str, workload: str = '', fast_check: bool = False, timeout: int = None):
+    def _verify_elasticsearch_data_uploaded(self, index: str, uuid: str, workload: str = '', fast_check: bool = False, timeout: int = None, es_fetch_min_time: int = None):
         """
         This method verify that elasticsearch data uploaded
         :param index:
@@ -273,9 +282,9 @@ class BenchmarkOperatorWorkloadsOperations:
         :return: ids
         """
         if workload:
-            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=workload, fast_check=fast_check, timeout=timeout)
+            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=workload, fast_check=fast_check, timeout=timeout, es_fetch_min_time=es_fetch_min_time)
         else:
-            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=self._workload_name, fast_check=fast_check, timeout=timeout)
+            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=self._workload_name, fast_check=fast_check, timeout=timeout, es_fetch_min_time=es_fetch_min_time)
 
     def _upload_workload_to_elasticsearch(self, index: str, kind: str, status: str, result: dict = None):
         """
@@ -306,7 +315,7 @@ class BenchmarkOperatorWorkloadsOperations:
         vm_name = ''
         for label in labels:
             vm_name = self._oc.get_vm(label=label)
-            self._oc.save_vm_log(vm_name=vm_name)
+            self._virtctl.save_vm_log(vm_name=vm_name)
         return vm_name
 
     def _create_pod_log(self, label: str = '', database: str = '') -> str:
@@ -448,16 +457,27 @@ class BenchmarkOperatorWorkloadsOperations:
             if not self._oc.is_odf_installed():
                 raise ODFNonInstalled()
 
+    @logger_time_stamp
+    def verify_lso(self):
+        """
+        This method Verifies that lso path exist
+        :return:
+        """
+        if not self._lso_path:
+            raise EmptyLSOPath()
+
     @logger_time_stamp
     def delete_all(self):
         """
-        This method delete all pod in namespace
+        This method delete all pod or unbound pv in namespace
         :return:
         """
         # make undeploy benchmark controller manager if exist
         self.make_undeploy_benchmark_controller_manager_if_exist(runner_path=environment_variables.environment_variables_dict['runner_path'])
         if 'hammerdb' in self._workload:
             self._oc.delete_namespace(namespace=f"{self._workload.split('_')[2]}-db")
+        if self._storage_type == 'lso':
+            self._oc.delete_available_released_pv()
 
     @logger_time_stamp
     def clear_nodes_cache(self):
@@ -473,10 +493,12 @@ class BenchmarkOperatorWorkloadsOperations:
         """
         self.delete_all()
         self.clear_nodes_cache()
-        # make deploy benchmark controller manager
-        self.make_deploy_benchmark_controller_manager(runner_path=environment_variables.environment_variables_dict['runner_path'])
         if self._odf_pvc:
             self.odf_pvc_verification()
+        if 'lso' in self._workload:
+            self.verify_lso()
+        # make deploy benchmark controller manager
+        self.make_deploy_benchmark_controller_manager(runner_path=environment_variables.environment_variables_dict['runner_path'])
         self._template.generate_yamls()
         if self._enable_prometheus_snapshot:
             self.start_prometheus()
