@@ -18,6 +18,8 @@ class ClusterBusterWorkloads(WorkloadsOperations):
         self.__clusterbuster_path = '/tmp/OpenShift4-tools/CI/./run-kata-perf-suite'
         # environment variables
         self.__namespace = self._environment_variables_dict.get('namespace', '')
+        self.__clusterbuster_workload = self._environment_variables_dict.get('clusterbuster_workload', '')
+        self.__clusterbuster_uuid = self._environment_variables_dict.get('clusterbuster_uuid', '')
         self.__result_report = '/tmp/clusterbuster-report.json'
         self.__artifactdir = os.path.join(self._run_artifacts_path, 'clusterbuster-ci')
         self.__clusterbuster_log = os.path.join(self._run_artifacts_path, 'clusterbuster.log')
@@ -91,7 +93,11 @@ class ClusterBusterWorkloads(WorkloadsOperations):
         This method run ClusterBuster workload
         :return:
         """
-        self.__ssh.run(cmd=f'{self.__clusterbuster_path} --run_type={self._run_type} --client-pin-node={self._pin_node1} --server-pin-node={self._pin_node2} --sync-pin-node={self._pin_node2} --force-cleanup-i-know-this-is-dangerous=600 --basename={self.__namespace} --artifactdir={self.__artifactdir} --analyze={self.__result_report} > {self.__clusterbuster_log} 2>&1')
+        # release pass uuid and workload
+        if self._run_type == 'release':
+            self.__ssh.run(cmd=f'{self.__clusterbuster_path} --run_type={self._run_type} --client-pin-node={self._pin_node1} --server-pin-node={self._pin_node2} --sync-pin-node={self._pin_node2} --force-cleanup-i-know-this-is-dangerous=600 --basename={self.__namespace} --artifactdir={self.__artifactdir} --analyze={self.__result_report} --uuid={self.__clusterbuster_uuid} {self.__clusterbuster_workload} > {self.__clusterbuster_log} 2>&1')
+        else:
+            self.__ssh.run(cmd=f'{self.__clusterbuster_path} --run_type={self._run_type} --client-pin-node={self._pin_node1} --server-pin-node={self._pin_node2} --sync-pin-node={self._pin_node2} --force-cleanup-i-know-this-is-dangerous=600 --basename={self.__namespace} --artifactdir={self.__artifactdir} --analyze={self.__result_report} > {self.__clusterbuster_log} 2>&1')
         # Check that Result file exist and not empty
         if os.path.exists(os.path.join(self.__result_report)) and not os.stat(self.__result_report).st_size == 0:
             self.__ssh.run(cmd=f'cp {self.__result_report} {self._run_artifacts_path}')
