@@ -12,7 +12,7 @@ from benchmark_runner.main.environment_variables import environment_variables
 
 class CreateKata(CreateOCPResourceOperations):
     """
-    This class is create OCP resources
+    This class is created Kata operator
     """
     def __init__(self, oc: OC, path: str, resource_list: list):
         super().__init__(oc)
@@ -29,9 +29,10 @@ class CreateKata(CreateOCPResourceOperations):
         :param resource: name of resource to create
         :return:
         """
+        timeout = int(environment_variables.environment_variables_dict['timeout'])
         current_wait_time = 0
-        while current_wait_time < int(environment_variables.environment_variables_dict['timeout']):
-            self.__oc._create_async(yaml_file)
+        while timeout <= 0 or current_wait_time < timeout:
+            self.__oc.create_async(yaml_file)
             # We cannot wait for a condition here, because the
             # create_async may simply not work even if it returns success.
             time.sleep(OC.SLEEP_TIME)
@@ -50,7 +51,7 @@ class CreateKata(CreateOCPResourceOperations):
             logger.info(f'run {resource}')
             if '01_operator.yaml' == resource:
                 # Wait for kataconfig CRD to exist
-                self.__oc._create_async(os.path.join(self.__path, resource))
+                self.__oc.create_async(os.path.join(self.__path, resource))
                 self.wait_for_ocp_resource_create(resource='kata',
                                                   verify_cmd='if oc get crd kataconfigs.kataconfiguration.openshift.io >/dev/null 2>&1 ; then echo succeeded ; fi',
                                                   status='succeeded')
@@ -70,6 +71,6 @@ class CreateKata(CreateOCPResourceOperations):
                 completed_nodes_count = self.__oc.run(cmd="oc get kataconfig -ojsonpath='{.items[0].status.installationStatus.completed.completed_nodes_count}'")
                 if total_nodes_count != completed_nodes_count:
                     raise KataInstallationFailed(f'not all nodes installed successfully total {total_nodes_count} != completed {completed_nodes_count}')
-            elif '03_ocp48_patch.sh' == resource:
+            elif resource.endswith('.sh'):
                 self.__oc.run(cmd=f'chmod +x {os.path.join(self.__path, resource)}; {os.path.join(self.__path, resource)}')
         return True
