@@ -8,7 +8,7 @@ from benchmark_runner.common.ocp_resources.create_ocp_resource_operations import
 
 class CreateLocalStorage(CreateOCPResourceOperations):
     """
-    This class is create OCP resources
+    This class is created Local Storage operator
     """
     def __init__(self, oc: OC, path: str, resource_list: list):
         super().__init__(oc)
@@ -24,7 +24,8 @@ class CreateLocalStorage(CreateOCPResourceOperations):
         """
         for resource in self.__resource_list:
             logger.info(f'run {resource}')
-            self.__oc._create_async(yaml=os.path.join(self.__path, resource))
+            self.__oc.create_async(yaml=os.path.join(self.__path, resource))
+
         # verify once after create all resource files
         self.wait_for_ocp_resource_create(resource='local_storage',
                                           verify_cmd="oc -n openshift-local-storage wait deployment/local-storage-operator --for=condition=Available",
