@@ -1,11 +1,12 @@
 
 import os
+import time
 import yaml
 import sys
 import benchmark_runner
 from multiprocessing import Process
 
-from jinja2 import Template
+from jinja2 import Template, TemplateSyntaxError
 from benchmark_runner.common.template_operations.render_yaml_from_template import render_yaml_file
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp, logger
 from benchmark_runner.main.environment_variables import environment_variables
@@ -21,12 +22,23 @@ class TemplateOperations:
     def __initialize_dependent_variables__(self):
         self.__run_type = self.__environment_variables_dict.get('run_type', '')
         self.__run_artifacts_path = self.__environment_variables_dict.get('run_artifacts_path', '')
+        self.__bulk_sleep_time = int(self.__environment_variables_dict.get('bulk_sleep_time', ''))
         self.__dir_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'templates'))
         if self.__run_type == 'test_ci':
             self.__environment_variables_dict['es_suffix'] = '-test-ci'
         else:
             self.__environment_variables_dict['es_suffix'] = ''
 
+    @logger_time_stamp
+    def __split_run_bulks(self, iterable: range, limit: int = 1):
+        """
+        This method splits run into bulk depends on threads limit
+        @return: run bulks
+        """
+        length = len(iterable)
+        for ndx in range(0, length, limit):
+            yield iterable[ndx:min(ndx + limit, length)]
+
     def __get_workload_template_kind(self):
         """
         Kata shares templates with pod
@@ -82,16 +94,20 @@ class TemplateOperations:
                 **extra_data, **extra_kind_data, **extra_runtype_data, **extra_kind_runtype_data}
 
     @logger_time_stamp
-    def generate_yamls_internal(self, scale: str = None, scale_node: str = None):
+    def __generate_yamls_internal(self, scale: str = None, scale_num: str = None, scale_node: str = None, redis: str = None):
         """
         Generate required .yaml files as a dictionary of file names and contents
-        :return: Dictionary <filename:contents>
+        @param scale: current scale 0,1,2
+        @param scale_num: the scale number
+        @param scale_node: the scale node
+        @param redis: redis
         """
-        # This really belongs in __init__, but some of the tests rely on
-        # being able to create this without an actual workload.
+        # This really belongs in __init__, but some tests rely on
+        # being able to create this without an actual workload
+        answer = {}
         self.__workload_name = self.__workload.split('_')[0]
         self.__workload_kind = self.__workload.split('_')[1]
-        self.__workload_extra_name = '_'.join(self.__workload.split('_')[2:])
+        self.__workload_extra_name = '_'.join(self.__workload.split('_')[2:3])
         self.__workload_template_kind = self.__get_workload_template_kind()
         if self.__workload_extra_name:
             self.__standard_output_file = f"{'_'.join([self.__workload_name, self.__workload_template_kind, self.__workload_extra_name])}.yaml"
@@ -112,6 +128,7 @@ class TemplateOperations:
             'standard_output_file': self.__standard_output_file,
             'standard_template_file': self.__standard_template_file,
             'scale': scale,
+            'scale_num': scale_num,
             'scale_node': scale_node
             }
         self.__environment_variables_dict = {**self.__environment_variables_dict, **template_render_data}
@@ -129,7 +146,6 @@ class TemplateOperations:
 
         render_data = self.__build_template_data(template_render_data, workload_data)
 
-        answer = {}
         out_files = [{'name': self.__standard_output_file, 'template': self.__standard_template_file}]
         if 'files' in workload_data:
             out_files.extend(workload_data['files'])
@@ -140,44 +156,49 @@ class TemplateOperations:
             else:
                 file_components = os.path.splitext(filename)
                 template_file = f'{file_components[0]}_template{file_components[1]}'
-            with open(os.path.join(workload_dir_path, 'internal_data', template_file)) as f:
-                template = Template(f.read())
+            try:
+                with open(os.path.join(workload_dir_path, 'internal_data', template_file)) as f:
+                    template = Template(f.read())
+            except TemplateSyntaxError as err:
+                raise SyntaxError(f"Error while rendering {template_file}: {err}")
             answer[filename] = f"{template.render(render_data)}\n"
+            # namespace
+            if os.path.isfile(os.path.join(workload_dir_path, 'internal_data', f'namespace_template.yaml')):
+                answer['namespace.yaml'] = render_yaml_file(dir_path=os.path.join(workload_dir_path, 'internal_data'), yaml_file='namespace_template.yaml', environment_variable_dict=self.__environment_variables_dict)
             if scale:
-                answer['redis.yaml'] = render_yaml_file(dir_path=os.path.join(self.__dir_path, 'scale'), yaml_file='redis.yaml', environment_variable_dict=self.__environment_variables_dict)
-                answer['state_signals_exporter_pod.yaml'] = render_yaml_file(dir_path=os.path.join(self.__dir_path, 'scale'), yaml_file='state_signals_exporter_pod.yaml', environment_variable_dict=self.__environment_variables_dict)
-        return answer
-
-    @logger_time_stamp
-    def generate_files(self, data_files: dict):
-        for filename, data in data_files.items():
+                answer['namespace.yaml'] = render_yaml_file(dir_path=os.path.join(self.__dir_path, 'scale'), yaml_file='namespace.yaml', environment_variable_dict=self.__environment_variables_dict)
+                if redis:
+                    answer['redis.yaml'] = render_yaml_file(dir_path=os.path.join(self.__dir_path, 'scale'), yaml_file='redis.yaml', environment_variable_dict=self.__environment_variables_dict)
+                    answer['state_signals_exporter_pod.yaml'] = render_yaml_file(dir_path=os.path.join(self.__dir_path, 'scale'), yaml_file='state_signals_exporter_pod.yaml', environment_variable_dict=self.__environment_variables_dict)
+        for filename, data in answer.items():
             with open(os.path.join(self.__run_artifacts_path, filename), 'w') as f:
                 f.write(data)
 
     @logger_time_stamp
-    def generate_yamls(self, scale: str = '', scale_nodes: list = []):
+    def generate_yamls(self, scale: str = '', scale_nodes: list = [], redis: str = None, thread_limit: int = None):
         """
         This method generate workload yaml from template
         :return:
         """
         if not scale:
-            self.generate_files(self.generate_yamls_internal())
+            self.__generate_yamls_internal()
         else:
             proc = []
             scale = int(scale)
+            bulks = tuple(self.__split_run_bulks(iterable=range(scale * len(scale_nodes)), limit=thread_limit))
             if scale_nodes:
-                count = 0
-                for scale_node in range(len(scale_nodes)):
-                    for scale_num in range(scale):
-                        count += 1
-                        p = Process(target=self.generate_files, args=(self.generate_yamls_internal(scale=str(count), scale_node=scale_nodes[scale_node]), ))
+                for bulk in bulks:
+                    for num in bulk:
+                        p = Process(target=self.__generate_yamls_internal, args=(str(num), str(scale * len(scale_nodes)), scale_nodes[int(num / scale)], redis,))
                         p.start()
                         proc.append(p)
                     for p in proc:
                         p.join()
+                    # sleep between bulks
+                    time.sleep(self.__bulk_sleep_time)
             else:
                 for scale_num in range(scale):
-                    self.generate_files(self.generate_yamls_internal(scale=str(scale_num+1)))
+                    self.__generate_yamls_internal(scale=str(scale_num))
 
     # The following routines are for testing purposes,
     # in particular to allow a known environment to be set up for golden file testing
