@@ -1,14 +1,16 @@
 
 import ast
 import os
+import time
 import datetime
 import tarfile
 import shutil
 from csv import DictReader
 
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp
-from benchmark_runner.workloads.workloads_exceptions import ODFNonInstalled
+from benchmark_runner.workloads.workloads_exceptions import ODFNonInstalled, MissingScaleNodes, MissingRedis
 from benchmark_runner.common.oc.oc import OC
+from benchmark_runner.common.virtctl.virtctl import Virtctl
 from benchmark_runner.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
 from benchmark_runner.main.environment_variables import environment_variables
 from benchmark_runner.common.clouds.shared.s3.s3_operations import S3Operations
@@ -20,6 +22,9 @@ from benchmark_runner.common.clouds.IBM.ibm_operations import IBMOperations
 
 class WorkloadsOperations:
     oc = None
+    MILLISECONDS = 1000
+    REPEAT_TIMES = 3
+    SLEEP_TIME = 3
     """
     This class run workloads
     """
@@ -50,10 +55,22 @@ class WorkloadsOperations:
         self._es_password = self._environment_variables_dict.get('elasticsearch_password', '')
         self._es_url_protocol = self._environment_variables_dict['elasticsearch_url_protocol']
         self._scale = self._environment_variables_dict.get('scale', '')
+        self._redis = self._environment_variables_dict.get('redis', '')
+        self._threads_limit = self._environment_variables_dict.get('threads_limit', '')
         if self._scale:
             self._scale = int(self._scale)
             self._scale_nodes = self._environment_variables_dict.get('scale_nodes', '')
+            self._redis = self._environment_variables_dict.get('redis', '')
+            if not self._scale_nodes:
+                raise MissingScaleNodes()
+            if not self._redis and 'vdbench' in self._workload:
+                raise MissingRedis()
             self._scale_node_list = ast.literal_eval(self._scale_nodes)
+            if self._threads_limit:
+                self._threads_limit = int(self._threads_limit)
+            else:
+                self._threads_limit = self._scale * len(self._scale_node_list)
+            self._bulk_sleep_time = int(self._environment_variables_dict.get('bulk_sleep_time', ''))
         else:
             self._scale_node_list = []
         self._timeout = int(self._environment_variables_dict.get('timeout', ''))
@@ -72,20 +89,22 @@ class WorkloadsOperations:
         if WorkloadsOperations.oc is None:
             WorkloadsOperations.oc = self.set_login(kubeadmin_password=self._kubeadmin_password)
         self._oc = WorkloadsOperations.oc
+        self._virtctl = Virtctl()
 
-        # PrometheusSnapshot
+        # Prometheus Snapshot
         if self._enable_prometheus_snapshot:
             self._snapshot = PrometheusSnapshot(oc=self._oc, artifacts_path=self._run_artifacts_path, verbose=True)
+        self._prometheus_snap_interval = self._environment_variables_dict.get('prometheus_snap_interval', '')
 
     def __get_workload_file_name(self, workload):
-            """
-            This method returns workload name
-            :return:
-            """
-            if self._scale:
-                return f'{workload}-scale-{self._time_stamp_format}'
-            else:
-                return f'{workload}-{self._time_stamp_format}'
+        """
+        This method returns workload name
+        :return:
+        """
+        if self._scale:
+            return f'{workload}-scale-{self._time_stamp_format}'
+        else:
+            return f'{workload}-{self._time_stamp_format}'
 
     def set_login(self, kubeadmin_password: str = ''):
         """
@@ -153,7 +172,7 @@ class WorkloadsOperations:
         vm_name = ''
         for label in labels:
             vm_name = self._oc.get_vm(label=label)
-            self._oc.save_vm_log(vm_name=vm_name)
+            self._virtctl.save_vm_log(vm_name=vm_name)
         return vm_name
 
     def _create_pod_log(self, pod: str = ''):
@@ -208,7 +227,7 @@ class WorkloadsOperations:
         """
         This method create pod run artifacts
         :param pod_name: pod name
-        :return: run results dict
+        :return: run results list of dicts
         """
         result_list = []
         pod_log_file = self._create_pod_log(pod=pod_name)
@@ -329,18 +348,16 @@ class WorkloadsOperations:
                     'ci_date': datetime.datetime.now().strftime(date_format),
                     'uuid': self._uuid,
                     'pin_node1': self._pin_node1,
-                    'pin_node2': self._pin_node2}
+                    'pin_node2': self._pin_node2,
+                    'odf_disk_count': self._oc.get_odf_disk_count()}
         if kind:
             metadata.update({'kind': kind})
         if status:
             metadata.update({'run_status': status})
         if self._scale:
             metadata.update({'scale': int(self._scale)*len(self._scale_node_list)})
-            count = 0
-            for scale_node in range(len(self._scale_node_list)):
-                for scale_num in range(self._scale):
-                    count += 1
-                    metadata.update({f'scale-{kind}--node-{count}': self._scale_node_list[scale_node]})
+        if 'bootstorm' in self._workload:
+            metadata.update({'vm_os_version': 'fedora37'})
         if result:
             metadata.update(result)
 
@@ -367,13 +384,14 @@ class WorkloadsOperations:
         self._es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid)
 
     @logger_time_stamp
-    def update_ci_status(self, status: str, ci_minutes_time: int, benchmark_operator_id: str, benchmark_wrapper_id: str, ocp_install_minutes_time: int = 0, ocp_resource_install_minutes_time: int = 0):
+    def update_ci_status(self, status: str, ci_minutes_time: int, benchmark_runner_id: str, benchmark_operator_id: str, benchmark_wrapper_id: str, ocp_install_minutes_time: int = 0, ocp_resource_install_minutes_time: int = 0):
         """
         This method update ci status Pass/Failed
         :param status: Pass/Failed
         :param ci_minutes_time: ci time in minutes
-        :param benchmark_wrapper_id: benchmark_wrapper last repository commit id
+        :param benchmark_runner_id: benchmark_runner last repository commit id
         :param benchmark_operator_id: benchmark_operator last repository commit id
+        :param benchmark_wrapper_id: benchmark_wrapper last repository commit id
         :param ocp_install_minutes_time: ocp install minutes time, default 0 because ocp install run once a week
         :param ocp_resource_install_minutes_time: ocp install minutes time, default 0 because ocp install run once a week
         :return:
@@ -389,14 +407,51 @@ class WorkloadsOperations:
             ibm_operations.ibm_connect()
             ocp_install_minutes_time = ibm_operations.get_ocp_install_time()
             ibm_operations.ibm_disconnect()
-        metadata.update({'status': status, 'status#': status_dict[status], 'ci_minutes_time': ci_minutes_time, 'benchmark_operator_id': benchmark_operator_id, 'benchmark_wrapper_id': benchmark_wrapper_id, 'ocp_install_minutes_time': ocp_install_minutes_time, 'ocp_resource_install_minutes_time': ocp_resource_install_minutes_time})
+        metadata.update({'status': status, 'status#': status_dict[status], 'ci_minutes_time': ci_minutes_time, 'benchmark_runner_id': benchmark_runner_id, 'benchmark_operator_id': benchmark_operator_id, 'benchmark_wrapper_id': benchmark_wrapper_id, 'ocp_install_minutes_time': ocp_install_minutes_time, 'ocp_resource_install_minutes_time': ocp_resource_install_minutes_time})
         self._es_operations.upload_to_elasticsearch(index=es_index, data=metadata)
 
+    @logger_time_stamp
+    def split_run_bulks(self, iterable: range, limit: int = 1):
+        """
+        This method splits run into bulk depends on threads limit
+        @return: run bulks
+        """
+        length = len(iterable)
+        for ndx in range(0, length, limit):
+            yield iterable[ndx:min(ndx + limit, length)]
+
+    @staticmethod
+    @logger_time_stamp
+    def parse_prometheus_metrics(data):
+        """
+        This method parse prometheus metrics and return summary result
+        @return:
+        """
+        result_dict = {}
+        for query, data_list in data.items():
+            if 'containerCPU-benchmark-runner' in query:
+                suffix = 'CPU'
+            elif 'containerMemory-benchmark-runner' in query:
+                suffix = 'Memory'
+            total = 0
+            max = 0
+            for item in data_list:
+                for val in item['values']:
+                    if float(val[1]) > max:
+                        max = round(float(val[1]), 3)
+                total = total + max
+                result_dict[f"{item['metric']['node']}_{suffix}"] = max
+                result_dict[f'total_{suffix}'] = total
+        return result_dict
+
     @logger_time_stamp
     def clear_nodes_cache(self):
         """
         This method clear nodes cache
         """
+        for i in range(self.REPEAT_TIMES-1):
+            self._oc.clear_node_caches()
+            time.sleep(self.SLEEP_TIME)
         self._oc.clear_node_caches()
 
     def initialize_workload(self):
@@ -408,7 +463,7 @@ class WorkloadsOperations:
         self.clear_nodes_cache()
         if self._odf_pvc:
             self.odf_pvc_verification()
-        self._template.generate_yamls(scale=str(self._scale), scale_nodes=self._scale_node_list)
+        self._template.generate_yamls(scale=str(self._scale), scale_nodes=self._scale_node_list, redis=self._redis, thread_limit=self._threads_limit)
         if self._enable_prometheus_snapshot:
             self.start_prometheus()
 
